@@ -45,6 +45,7 @@
 #  include "config.h"
 #endif
 
+#include "../gst-i18n-lib.h"
 #include "gstspider.h"
 #include "gstspideridentity.h"
 #include "gstsearchfuncs.h"
@@ -65,11 +66,12 @@ enum {
 };
 
 /* generic templates */
-GST_PAD_TEMPLATE_FACTORY (spider_src_factory,
+static GstStaticPadTemplate spider_src_factory =
+GST_STATIC_PAD_TEMPLATE (
   "src_%d",
   GST_PAD_SRC,
   GST_PAD_REQUEST,
-  NULL      /* no caps */
+  GST_STATIC_CAPS_ANY
 );
 
 /* standard GObject stuff */
@@ -152,7 +154,8 @@ gst_spider_class_init (GstSpiderClass *klass)
   gobject_class->get_property = gst_spider_get_property;
   gobject_class->dispose = gst_spider_dispose;
 
-  gst_element_class_add_pad_template (gstelement_class, GST_PAD_TEMPLATE_GET (spider_src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&spider_src_factory));
   gst_element_class_set_details (gstelement_class, &gst_spider_details);
   
   gstelement_class->request_new_pad = GST_DEBUG_FUNCPTR(gst_spider_request_new_pad);
@@ -209,6 +212,7 @@ gst_spider_request_new_pad (GstElement *element, GstPadTemplate *templ, const gc
   gst_bin_add (GST_BIN (element), GST_ELEMENT (identity));
   
   returnpad = gst_element_add_ghost_pad (element, returnpad, padname);
+  g_free (padname);
   gst_spider_link_new (identity);
   GST_DEBUG ("successuflly created requested pad %s:%s", GST_DEBUG_PAD_NAME (returnpad));
   
@@ -384,6 +388,42 @@ gst_spider_identity_plug (GstSpiderIdentity *ident)
   /* return if we're already plugged */
   if (ident->plugged) return;
 
+  /* check if there is at least one element factory that can handle the
+     identity's src caps */
+  {
+    GstCaps *src_caps = gst_pad_get_caps (ident->src);
+    if (! gst_caps_is_empty (src_caps) && ! gst_caps_is_any (src_caps))
+    {
+      const char *mime;
+      GList *factories;
+      GstPadTemplate *padtemp;
+      gboolean found = FALSE;
+
+      mime = gst_structure_get_name (gst_caps_get_structure (src_caps, 0));
+
+      factories = spider->factories;
+      while (factories)
+      {
+        if ((padtemp = gst_autoplug_can_connect_src (factories->data, src_caps)))
+        {
+          const GstCaps *caps = gst_pad_template_get_caps (padtemp);
+          GST_DEBUG ("can connect src to pad template: %" GST_PTR_FORMAT, caps);
+          found = TRUE;
+        }
+        factories = factories->next;
+      }
+      if (!found)
+      {
+        GST_ELEMENT_ERROR (spider, STREAM, CODEC_NOT_FOUND,
+                           (_("There is no element present to handle the stream's mime type %s."), mime),
+                           (NULL));
+	return;
+      }
+    }
+  }
+
+
+
   /* get the direction of our ident */
   if (GST_PAD_PEER (ident->sink))
   {
@@ -586,6 +626,8 @@ gst_spider_plug_from_srcpad (GstSpiderConnection *conn, GstPad *srcpad)
   gboolean result = TRUE;
   GstSpider *spider = (GstSpider *) GST_OBJECT_PARENT (conn->src);
   GstElement *startelement = conn->current;
+  GstCaps *caps1;
+  GstCaps *caps2;
 
   g_assert ((GstElement *) GST_OBJECT_PARENT (srcpad) == conn->current);
   GST_DEBUG ("trying to plug from %s:%s to %s", 
@@ -601,7 +643,11 @@ gst_spider_plug_from_srcpad (GstSpiderConnection *conn, GstPad *srcpad)
   }
       
   /* find a path from src to sink */
-  plugpath = gst_autoplug_sp (gst_pad_get_caps (srcpad), gst_pad_get_caps (conn->src->sink), spider->factories);
+  caps1 = gst_pad_get_caps (srcpad);
+  caps2 = gst_pad_get_caps (conn->src->sink);
+  plugpath = gst_autoplug_sp (caps1, caps2, spider->factories);
+  gst_caps_free (caps1);
+  gst_caps_free (caps2);
   
   /* prints out the path that was found for plugging */
   /* g_print ("found path from %s to %s:\n", GST_ELEMENT_NAME (conn->current), GST_ELEMENT_NAME (conn->src));
