@@ -27,7 +27,9 @@
 #  include "config.h"
 #endif
 
+#include "../gst-i18n-lib.h"
 #include "gstidentity.h"
+#include <gst/gstmarshal.h>
 
 GST_DEBUG_CATEGORY_STATIC (gst_identity_debug);
 #define GST_CAT_DEFAULT gst_identity_debug
@@ -56,47 +58,22 @@ enum {
   ARG_DROP_PROBABILITY,
   ARG_SILENT,
   ARG_LAST_MESSAGE,
-  ARG_DUMP,
-  ARG_DELAY_CAPSNEGO,
+  ARG_DUMP
 };
 
 
-static void gst_identity_base_init	(gpointer g_class);
-static void gst_identity_class_init	(GstIdentityClass *klass);
-static void gst_identity_init		(GstIdentity *identity);
+#define _do_init(bla) \
+    GST_DEBUG_CATEGORY_INIT (gst_identity_debug, "identity", 0, "identity element");
+
+GST_BOILERPLATE_FULL (GstIdentity, gst_identity, GstElement, GST_TYPE_ELEMENT, _do_init);
 
 static void gst_identity_set_property	(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void gst_identity_get_property	(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
 static void gst_identity_chain		(GstPad *pad, GstData *_data);
 
-static GstElementClass *parent_class = NULL;
 static guint gst_identity_signals[LAST_SIGNAL] = { 0 };
 
-GType
-gst_identity_get_type (void) 
-{
-  static GType identity_type = 0;
-
-  if (!identity_type) {
-    static const GTypeInfo identity_info = {
-      sizeof(GstIdentityClass),
-      gst_identity_base_init,
-      NULL,
-      (GClassInitFunc)gst_identity_class_init,
-      NULL,
-      NULL,
-      sizeof(GstIdentity),
-      0,
-      (GInstanceInitFunc)gst_identity_init,
-    };
-    identity_type = g_type_register_static (GST_TYPE_ELEMENT, "GstIdentity", &identity_info, 0);
-  
-    GST_DEBUG_CATEGORY_INIT (gst_identity_debug, "identity", 0, "identity element");
-  }
-  return identity_type;
-}
-
 static void
 gst_identity_base_init (gpointer g_class)
 {
@@ -111,7 +88,6 @@ gst_identity_class_init (GstIdentityClass *klass)
 
   gobject_class = G_OBJECT_CLASS (klass);
 
-  parent_class = g_type_class_peek_parent (klass);
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_LOOP_BASED,
     g_param_spec_boolean ("loop-based", "Loop-based", 
@@ -138,9 +114,6 @@ gst_identity_class_init (GstIdentityClass *klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DUMP,
     g_param_spec_boolean("dump", "Dump", "Dump buffer contents",
                          FALSE, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DELAY_CAPSNEGO,
-    g_param_spec_boolean("delay_capsnego", "Delay Caps Nego", "Delay capsnegotiation to loop/chain function",
-                         FALSE, G_PARAM_READWRITE));
 
   gst_identity_signals[SIGNAL_HANDOFF] =
     g_signal_new ("handoff", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
@@ -152,72 +125,19 @@ gst_identity_class_init (GstIdentityClass *klass)
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_identity_get_property);
 }
 
-static GstBufferPool*
-gst_identity_get_bufferpool (GstPad *pad)
-{
-  GstIdentity *identity;
-
-  identity = GST_IDENTITY (gst_pad_get_parent (pad));
-
-  return gst_pad_get_bufferpool (identity->srcpad);
-}
-
-static GstCaps*
-gst_identity_getcaps (GstPad *pad, GstCaps *caps)
-{
-  GstIdentity *identity;
-  GstPad *otherpad;
-  
-  identity = GST_IDENTITY (gst_pad_get_parent (pad));
-
-  if (identity->delay_capsnego) {
-    return NULL;
-  }
-
-  otherpad = (pad == identity->srcpad ? identity->sinkpad : identity->srcpad);
-
-  return gst_pad_get_allowed_caps (otherpad);
-}
-
-static GstPadLinkReturn
-gst_identity_link (GstPad *pad, GstCaps *caps)
-{
-  GstIdentity *identity;
-  
-  identity = GST_IDENTITY (gst_pad_get_parent (pad));
-
-  if (GST_CAPS_IS_FIXED (caps)) {
-    if (identity->delay_capsnego && GST_PAD_IS_SINK (pad)) {
-      identity->srccaps = gst_caps_ref (caps);
-
-      return GST_PAD_LINK_OK;
-    }
-    else {
-      GstPad *otherpad;
-      
-      otherpad = (pad == identity->srcpad ? identity->sinkpad : identity->srcpad);
-
-      return gst_pad_try_set_caps (otherpad, caps);
-    }
-  }
-  else
-    return GST_PAD_LINK_DELAYED;
-}
-
 static void 
 gst_identity_init (GstIdentity *identity) 
 {
   identity->sinkpad = gst_pad_new ("sink", GST_PAD_SINK);
   gst_element_add_pad (GST_ELEMENT (identity), identity->sinkpad);
   gst_pad_set_chain_function (identity->sinkpad, GST_DEBUG_FUNCPTR (gst_identity_chain));
-  gst_pad_set_bufferpool_function (identity->sinkpad, gst_identity_get_bufferpool);
-  gst_pad_set_link_function (identity->sinkpad, gst_identity_link);
-  gst_pad_set_getcaps_function (identity->sinkpad, gst_identity_getcaps);
+  gst_pad_set_link_function (identity->sinkpad, gst_pad_proxy_pad_link);
+  gst_pad_set_getcaps_function (identity->sinkpad, gst_pad_proxy_getcaps);
   
   identity->srcpad = gst_pad_new ("src", GST_PAD_SRC);
   gst_element_add_pad (GST_ELEMENT (identity), identity->srcpad);
-  gst_pad_set_link_function (identity->srcpad, gst_identity_link);
-  gst_pad_set_getcaps_function (identity->srcpad, gst_identity_getcaps);
+  gst_pad_set_link_function (identity->srcpad, gst_pad_proxy_pad_link);
+  gst_pad_set_getcaps_function (identity->srcpad, gst_pad_proxy_getcaps);
 
   identity->loop_based = FALSE;
   identity->sleep_time = 0;
@@ -227,7 +147,6 @@ gst_identity_init (GstIdentity *identity)
   identity->silent = FALSE;
   identity->dump = FALSE;
   identity->last_message = NULL;
-  identity->delay_capsnego = FALSE;
   identity->srccaps = NULL;
 }
 
@@ -244,21 +163,13 @@ gst_identity_chain (GstPad *pad, GstData *_data)
 
   identity = GST_IDENTITY (gst_pad_get_parent (pad));
 
-  if (identity->delay_capsnego && identity->srccaps) {
-    if (gst_pad_try_set_caps (identity->srcpad, identity->srccaps) <= 0) {
-      if (!gst_pad_recover_caps_error (identity->srcpad, identity->srccaps)) {
-        gst_buffer_unref (buf);
-        return;
-      }
-    }
-    identity->srccaps = NULL;
-  }
-
   if (identity->error_after >= 0) {
     identity->error_after--;
     if (identity->error_after == 0) {
       gst_buffer_unref (buf);
-      gst_element_error (GST_ELEMENT (identity), "errored after iterations as requested");
+      GST_ELEMENT_ERROR (identity, CORE, FAILED,
+                           (_("Failed after iterations as requested.")),
+                           (NULL));
       return;
     }
   }
@@ -363,9 +274,6 @@ gst_identity_set_property (GObject *object, guint prop_id, const GValue *value,
     case ARG_DUMP:
       identity->dump = g_value_get_boolean (value);
       break;
-    case ARG_DELAY_CAPSNEGO:
-      identity->delay_capsnego = g_value_get_boolean (value);
-      break;
     case ARG_ERROR_AFTER:
       identity->error_after = g_value_get_int (value);
       break;
@@ -407,9 +315,6 @@ static void gst_identity_get_property(GObject *object, guint prop_id, GValue *va
     case ARG_DUMP:
       g_value_set_boolean (value, identity->dump);
       break;
-    case ARG_DELAY_CAPSNEGO:
-      g_value_set_boolean (value, identity->delay_capsnego);
-      break;
     case ARG_LAST_MESSAGE:
       g_value_set_string (value, identity->last_message);
       break;
