@@ -2,7 +2,7 @@
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
  *                    2000 Wim Taymans <wtay@chello.be>
  *
- * gstpipefilter.c: 
+ * gstpipefilter.c:
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,6 +26,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <stdio.h>
+#include <string.h>
 #include <unistd.h>
 #include <errno.h>
 
@@ -33,6 +34,7 @@
 #  include "config.h"
 #endif
 
+#include "../gst-i18n-lib.h"
 #include "gstpipefilter.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_pipefilter_debug);
@@ -59,9 +61,10 @@ enum {
 };
 
 
-static void			gst_pipefilter_base_init	(gpointer g_class);
-static void 			gst_pipefilter_class_init	(GstPipefilterClass *klass);
-static void 			gst_pipefilter_init		(GstPipefilter *pipefilter);
+#define _do_init(bla) \
+    GST_DEBUG_CATEGORY_INIT (gst_pipefilter_debug, "pipefilter", 0, "pipefilter element");
+
+GST_BOILERPLATE_FULL (GstPipefilter, gst_pipefilter, GstElement, GST_TYPE_ELEMENT, _do_init);
 
 static void 			gst_pipefilter_set_property	(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void 			gst_pipefilter_get_property	(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
@@ -72,33 +75,6 @@ static gboolean 		gst_pipefilter_handle_event 	(GstPad *pad, GstEvent *event);
 
 static GstElementStateReturn 	gst_pipefilter_change_state	(GstElement *element);
 
-static GstElementClass *parent_class = NULL;
-/*static guint gst_pipefilter_signals[LAST_SIGNAL] = { 0 };*/
-
-GType
-gst_pipefilter_get_type (void)
-{
-  static GType pipefilter_type = 0;
-
-  if (!pipefilter_type) {
-    static const GTypeInfo pipefilter_info = {
-      sizeof(GstPipefilterClass),
-      gst_pipefilter_base_init,
-      NULL,
-      (GClassInitFunc)gst_pipefilter_class_init,
-      NULL,
-      NULL,
-      sizeof(GstPipefilter),
-      0,
-      (GInstanceInitFunc)gst_pipefilter_init,
-    };
-    pipefilter_type = g_type_register_static(GST_TYPE_ELEMENT, "GstPipefilter", &pipefilter_info, 0);
-  
-    GST_DEBUG_CATEGORY_INIT (gst_pipefilter_debug, "pipefilter", 0, "pipefilter element");
-  }
-  return pipefilter_type;
-}
-
 static void
 gst_pipefilter_base_init (gpointer g_class)
 {
@@ -115,7 +91,6 @@ gst_pipefilter_class_init (GstPipefilterClass *klass)
   gobject_class = (GObjectClass*)klass;
   gstelement_class = (GstElementClass*)klass;
 
-  parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
 
   gobject_class->set_property = gst_pipefilter_set_property;  
   gobject_class->get_property = gst_pipefilter_get_property;
@@ -135,7 +110,6 @@ gst_pipefilter_init (GstPipefilter *pipefilter)
   pipefilter->sinkpad = gst_pad_new ("sink", GST_PAD_SINK);
   gst_element_add_pad (GST_ELEMENT (pipefilter), pipefilter->sinkpad);
   gst_pad_set_chain_function (pipefilter->sinkpad, gst_pipefilter_chain);
-  gst_pad_set_event_function (pipefilter->sinkpad, gst_pipefilter_handle_event);
 
   pipefilter->srcpad = gst_pad_new ("src", GST_PAD_SRC);
   gst_element_add_pad (GST_ELEMENT (pipefilter), pipefilter->srcpad);
@@ -155,10 +129,19 @@ gst_pipefilter_handle_event (GstPad *pad, GstEvent *event)
   pipefilter = GST_PIPEFILTER (gst_pad_get_parent (pad));
 
   GST_DEBUG ("pipefilter: %s received event", GST_ELEMENT_NAME (pipefilter));
-  if (close (pipefilter->fdin[1]) < 0)
-    perror("close");
-  if (close (pipefilter->fdout[0]) < 0)
-    perror("close");
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_EOS:
+      if (close (pipefilter->fdin[1]) < 0)
+        perror("close");
+      if (close (pipefilter->fdout[0]) < 0)
+        perror("close");
+      break;
+    default:
+      break;
+  }
+
+  gst_pad_event_default (pad, event);
 
   return TRUE;
 }
@@ -186,8 +169,7 @@ gst_pipefilter_get (GstPad *pad)
   readbytes = read(pipefilter->fdout[0], GST_BUFFER_DATA(newbuf), pipefilter->bytes_per_read);
   GST_DEBUG ("read %ld bytes", readbytes);
   if (readbytes < 0) {
-    perror("read");
-    gst_element_error(GST_ELEMENT(pipefilter),"reading");
+    GST_ELEMENT_ERROR (pipefilter, RESOURCE, READ, (NULL), GST_ERROR_SYSTEM);
     return NULL;
   }
   /* if we didn't get as many bytes as we asked for, we're at EOF */
@@ -206,7 +188,7 @@ gst_pipefilter_get (GstPad *pad)
 static void
 gst_pipefilter_chain (GstPad *pad,GstData *_data)
 {
-  GstBuffer *buf = GST_BUFFER (_data);
+  GstBuffer *buf;
   GstPipefilter *pipefilter;
   glong writebytes;
   guchar *data;
@@ -214,10 +196,15 @@ gst_pipefilter_chain (GstPad *pad,GstData *_data)
 
   g_return_if_fail(pad != NULL);
   g_return_if_fail(GST_IS_PAD(pad));
-  g_return_if_fail(buf != NULL);
+
+  if (GST_IS_EVENT (_data)) {
+    gst_pipefilter_handle_event (pad, GST_EVENT (_data));
+    return;
+  }
 
   pipefilter = GST_PIPEFILTER (gst_pad_get_parent (pad));
 
+  buf = GST_BUFFER (_data);
   data = GST_BUFFER_DATA(buf);
   size = GST_BUFFER_SIZE(buf);
 
@@ -225,8 +212,7 @@ gst_pipefilter_chain (GstPad *pad,GstData *_data)
   writebytes = write(pipefilter->fdin[1],data,size);
   GST_DEBUG ("written %ld bytes", writebytes);
   if (writebytes < 0) {
-    perror("write");
-    gst_element_error(GST_ELEMENT(pipefilter),"writing");
+    GST_ELEMENT_ERROR (pipefilter, RESOURCE, WRITE, (NULL), GST_ERROR_SYSTEM);
     return;
   }
   gst_buffer_unref(buf);
@@ -281,8 +267,7 @@ gst_pipefilter_open_file (GstPipefilter *src)
 
   if((src->childpid = fork()) == -1)
   {
-    perror("fork");
-    gst_element_error(GST_ELEMENT(src),"forking");
+    GST_ELEMENT_ERROR (src, RESOURCE, TOO_LAZY, (NULL), GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -294,9 +279,8 @@ gst_pipefilter_open_file (GstPipefilter *src)
     dup2(src->fdin[0], STDIN_FILENO);  /* set the childs input stream */
     dup2(src->fdout[1], STDOUT_FILENO);  /* set the childs output stream */
     execvp(src->command[0], &src->command[0]);
-    /* will only reach if error */
-    perror("exec");
-    gst_element_error(GST_ELEMENT(src),"starting child process");
+    /* will only be reached if execvp has an error */
+    GST_ELEMENT_ERROR (src, RESOURCE, TOO_LAZY, (NULL), GST_ERROR_SYSTEM);
     return FALSE;
     
   }
