@@ -20,13 +20,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* this file makes too much noise for most debugging sessions */
-#define GST_DEBUG_FORCE_DISABLE
 #include "gst_private.h"
 
+#include "gstatomic_impl.h"
 #include "gstdata.h"
 #include "gstdata_private.h"
-#include "gstlog.h"
+#include "gstinfo.h"
 
 /**
  * gst_data_init:
@@ -100,20 +99,22 @@ gst_data_copy (const GstData *data)
 }
 
 /**
- * gst_data_needs_copy_on_write:
+ * gst_data_is_writable:
  * @data: a #GstData to copy
  *
- * Returns: TRUE if the given #GstData is potentially shared and needs to
+ * Query if the gstdata needs to be copied before it can safely be modified.
+ *
+ * Returns: FALSE if the given #GstData is potentially shared and needs to
  * be copied before it can be modified safely.
  */
 gboolean
-gst_data_needs_copy_on_write (GstData *data) 
+gst_data_is_writable (GstData *data) 
 {
   gint refcount;
 
   g_return_val_if_fail (data != NULL, FALSE);
 
-  GST_ATOMIC_INT_READ (&data->refcount, &refcount);
+  refcount = gst_atomic_int_read (&data->refcount);
 
   if (refcount == 1 && !GST_DATA_FLAG_IS_SET (data, GST_DATA_READONLY))
     return FALSE;
@@ -141,7 +142,7 @@ gst_data_copy_on_write (GstData *data)
 
   g_return_val_if_fail (data != NULL, NULL);
 
-  GST_ATOMIC_INT_READ (&data->refcount, &refcount);
+  refcount = gst_atomic_int_read (&data->refcount);
 
   if (refcount == 1 && !GST_DATA_FLAG_IS_SET (data, GST_DATA_READONLY))
     return GST_DATA (data);
@@ -186,7 +187,10 @@ gst_data_ref (GstData *data)
   g_return_val_if_fail (data != NULL, NULL);
   g_return_val_if_fail (GST_DATA_REFCOUNT_VALUE(data) > 0, NULL);
 
-  GST_ATOMIC_INT_INC (&data->refcount);
+  GST_CAT_LOG (GST_CAT_BUFFER, "%p %d->%d", data,
+               GST_DATA_REFCOUNT_VALUE (data), GST_DATA_REFCOUNT_VALUE (data) + 1);
+  
+  gst_atomic_int_inc (&data->refcount);
 
   return data;
 }
@@ -207,7 +211,10 @@ gst_data_ref_by_count (GstData *data, gint count)
   g_return_val_if_fail (count >= 0, NULL);
   g_return_val_if_fail (GST_DATA_REFCOUNT_VALUE(data) > 0, NULL);
 
-  GST_ATOMIC_INT_ADD (&data->refcount, count);
+  GST_CAT_LOG (GST_CAT_BUFFER, "%p %d->%d", data,
+               GST_DATA_REFCOUNT_VALUE (data), GST_DATA_REFCOUNT_VALUE (data) + count);
+
+  gst_atomic_int_add (&data->refcount, count);
 
   return data;
 }
@@ -230,11 +237,11 @@ gst_data_unref (GstData *data)
 
   g_return_if_fail (data != NULL);
 
-  GST_INFO (GST_CAT_BUFFER, "unref data %p, count before unref is %d", 
-            data, GST_DATA_REFCOUNT_VALUE (data));
+  GST_CAT_LOG (GST_CAT_BUFFER, "%p %d->%d", data,
+               GST_DATA_REFCOUNT_VALUE (data), GST_DATA_REFCOUNT_VALUE (data) - 1);
   g_return_if_fail (GST_DATA_REFCOUNT_VALUE (data) > 0);
 
-  GST_ATOMIC_INT_DEC_AND_TEST (&data->refcount, &zero);
+  zero = gst_atomic_int_dec_and_test (&data->refcount);
 
   /* if we ended up with the refcount at zero, free the data */
   if (zero) {
@@ -242,4 +249,3 @@ gst_data_unref (GstData *data)
       data->free (data); 
   }
 }
-
