@@ -35,7 +35,7 @@ typedef enum {
   GST_FORMAT_BYTES   	=  2,
   GST_FORMAT_TIME 	=  3,
   GST_FORMAT_BUFFERS	=  4,
-  GST_FORMAT_PERCENT	=  5,
+  GST_FORMAT_PERCENT	=  5
 } GstFormat;
 
 /* a percentage is always relative to 1000000 */
@@ -86,9 +86,10 @@ GstFormat	gst_format_get_by_nick		(const gchar *nick);
 gboolean	gst_formats_contains		(const GstFormat *formats, GstFormat format);
 
 /* query for format details */
-const GstFormatDefinition*	
+G_CONST_RETURN GstFormatDefinition*	
 		gst_format_get_details		(GstFormat format);
-const GList* 	gst_format_get_definitions 	(void);
+G_CONST_RETURN GList*
+		gst_format_get_definitions 	(void);
 
 G_END_DECLS
 
