@@ -30,7 +30,12 @@
 #ifdef HAVE_DLFCN_H
 #include <dlfcn.h>
 #endif
+#ifdef HAVE_PRINTF_EXTENSION
+#include <printf.h>
+#endif
+#include <stdio.h> /* fprintf */
 #include <unistd.h>
+#include <string.h> /* G_VA_COPY */
 #include "gstinfo.h"
 #include "gstlog.h"
 #include "gst_private.h"
@@ -71,6 +76,19 @@ static void	gst_debug_reset_threshold	(gpointer category,
 						 gpointer unused);
 static void	gst_debug_reset_all_thresholds	(void);
 
+#ifdef HAVE_PRINTF_EXTENSION
+static int _gst_info_printf_extension (FILE *stream, const struct printf_info *info,
+    const void *const *args);
+static int _gst_info_printf_extension_arginfo (const struct printf_info *info, size_t n,
+    int *argtypes);
+#endif
+
+struct _GstDebugMessage {
+  gchar *		message;
+  const gchar *		format;
+  va_list		arguments;
+};
+
 /* list of all name/level pairs from --gst-debug and GST_DEBUG */
 static GStaticMutex __level_name_mutex = G_STATIC_MUTEX_INIT;
 static GSList *__level_name = NULL;
@@ -112,7 +130,6 @@ GstDebugCategory *GST_CAT_BUFFER = NULL;
 GstDebugCategory *GST_CAT_CAPS = NULL;
 GstDebugCategory *GST_CAT_CLOCK = NULL;
 GstDebugCategory *GST_CAT_ELEMENT_PADS = NULL;
-GstDebugCategory *GST_CAT_ELEMENT_FACTORY = NULL;
 GstDebugCategory *GST_CAT_PADS = NULL;
 GstDebugCategory *GST_CAT_PIPELINE = NULL;
 GstDebugCategory *GST_CAT_PLUGIN_LOADING = NULL;
@@ -123,6 +140,7 @@ GstDebugCategory *GST_CAT_TYPES = NULL;
 GstDebugCategory *GST_CAT_XML = NULL;
 GstDebugCategory *GST_CAT_NEGOTIATION = NULL;
 GstDebugCategory *GST_CAT_REFCOUNTING = NULL;
+GstDebugCategory *GST_CAT_ERROR_SYSTEM = NULL;
 GstDebugCategory *GST_CAT_EVENT = NULL;
 GstDebugCategory *GST_CAT_PARAMS = NULL;
 GstDebugCategory *GST_CAT_CALL_TRACE = NULL;
@@ -139,6 +157,11 @@ void _gst_debug_init (void)
   gst_atomic_int_init (&__default_level, GST_LEVEL_DEFAULT);
   gst_atomic_int_init (&__use_color, 1);
 
+#ifdef HAVE_PRINTF_EXTENSION
+  register_printf_function (GST_PTR_FORMAT[0], _gst_info_printf_extension,
+      _gst_info_printf_extension_arginfo);
+#endif
+
   /* do NOT use a single debug function before this line has been run */
   GST_CAT_DEFAULT	= _gst_debug_category_new ("default", 
 				GST_DEBUG_UNDERLINE,
@@ -192,9 +215,6 @@ void _gst_debug_init (void)
   GST_CAT_ELEMENT_PADS	= _gst_debug_category_new ("GST_ELEMENT_PADS",
 				GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_RED,
 				NULL);
-  GST_CAT_ELEMENT_FACTORY = _gst_debug_category_new ("GST_ELEMENT_FACTORY",
-				GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_RED,
-				NULL);
   GST_CAT_PADS		= _gst_debug_category_new ("GST_PADS",
 				GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_RED,
 				NULL);
@@ -225,6 +245,10 @@ void _gst_debug_init (void)
   GST_CAT_REFCOUNTING	= _gst_debug_category_new ("GST_REFCOUNTING",
 				GST_DEBUG_BOLD | GST_DEBUG_FG_BLUE | GST_DEBUG_BG_GREEN,
 				NULL);
+  GST_CAT_ERROR_SYSTEM  = _gst_debug_category_new ("GST_ERROR_SYSTEM",
+                                GST_DEBUG_BOLD | GST_DEBUG_FG_RED | GST_DEBUG_BG_WHITE,
+                                NULL);
+
   GST_CAT_EVENT		= _gst_debug_category_new ("GST_EVENT",
 				GST_DEBUG_BOLD | GST_DEBUG_FG_WHITE | GST_DEBUG_BG_RED,
 				NULL);
@@ -257,16 +281,16 @@ void _gst_debug_init (void)
  */
 void gst_debug_log (GstDebugCategory *category, GstDebugLevel level,
 		    const gchar *file, const gchar *function, gint line,
-		    GObject *object, gchar *format, ...)
+		    GObject *object, const gchar *format, ...)
 {
   va_list var_args;
   
   va_start (var_args, format);
-  gst_debug_logv (category, level, file, function, line, object, format, var_args);
+  gst_debug_log_valist (category, level, file, function, line, object, format, var_args);
   va_end (var_args);
 }
 /**
- * gst_debug_logv:
+ * gst_debug_log_valist:
  * @category: category to log
  * @level: level of the message is in
  * @file: the file that emitted the message, usually the __FILE__ identifier
@@ -278,11 +302,11 @@ void gst_debug_log (GstDebugCategory *category, GstDebugLevel level,
  * 
  * Logs the given message using the currently registered debugging handlers.
  */
-void gst_debug_logv (GstDebugCategory *category, GstDebugLevel level,
-		     const gchar *file, const gchar *function, gint line,
-		     GObject *object, gchar *format, va_list args)
+void gst_debug_log_valist (GstDebugCategory *category, GstDebugLevel level,
+			   const gchar *file, const gchar *function, gint line,
+			   GObject *object, const gchar *format, va_list args)
 {
-  gchar *message;
+  GstDebugMessage message;
   LogFuncEntry *entry;
   GSList *handler;
 
@@ -291,15 +315,82 @@ void gst_debug_logv (GstDebugCategory *category, GstDebugLevel level,
   g_return_if_fail (function != NULL);
   g_return_if_fail (format != NULL);
 
-  message = g_strdup_vprintf (format, args);
+  message.message = NULL;
+  message.format = format;
+  G_VA_COPY (message.arguments, args);
+  
   handler = __log_functions;
   while (handler) {
     entry = handler->data;
     handler = g_slist_next (handler);
-    entry->func (category, level, file, function, line, object, message, entry->user_data);
+    entry->func (category, level, file, function, line, object, &message, entry->user_data);
   }
-  g_free (message);
+  g_free (message.message);
+  va_end (message.arguments);
 }
+/**
+ * gst_debug_message_get:
+ * @message: a debug message
+ *
+ * Gets the string representation of a GstDebugMessage. This function is used
+ * in debug handlers to extract the message.
+ */
+const gchar *
+gst_debug_message_get (GstDebugMessage *message)
+{
+  if (message->message == NULL) {
+    message->message = g_strdup_vprintf (message->format, message->arguments);
+  }
+  return message->message;
+}
+
+
+static gchar *
+gst_debug_print_object (gpointer ptr)
+{
+  GObject *object = (GObject *)ptr;
+
+#ifdef unused
+  /* This is a cute trick to detect unmapped memory, but is unportable,
+   * slow, screws around with madvise, and not actually that useful. */
+  {
+    int ret;
+
+    ret = madvise ((void *)((unsigned long)ptr & (~0xfff)), 4096, 0);
+    if (ret == -1 && errno == ENOMEM) {
+      buffer = g_strdup_printf ("%p (unmapped memory)", ptr);
+    }
+  }
+#endif
+
+  /* nicely printed object */
+  if (object == NULL) {
+    return g_strdup ("(NULL)");
+  }
+  if (*(GType *)ptr == GST_TYPE_CAPS) {
+    return gst_caps_to_string ((GstCaps *)ptr);
+  }
+  if (*(GType *)ptr == GST_TYPE_STRUCTURE) {
+    return gst_structure_to_string ((GstStructure *)ptr);
+  }
+#ifdef USE_POISONING
+  if (*(int *)ptr == 0xffffffff) {
+    return g_strdup_printf ("<poisoned@%p>", ptr);
+  }
+#endif
+  if (GST_IS_PAD (object) && GST_OBJECT_NAME (object)) {
+    return g_strdup_printf ("<%s:%s>", GST_DEBUG_PAD_NAME (object));
+  }
+  if (GST_IS_OBJECT (object) && GST_OBJECT_NAME (object)) {
+    return g_strdup_printf ("<%s>", GST_OBJECT_NAME (object));
+  }
+  if (G_IS_OBJECT (object)) {
+    return g_strdup_printf ("<%s@%p>", G_OBJECT_TYPE_NAME(object), object);  
+  }
+
+  return g_strdup_printf ("%p", ptr);
+}
+
 /**
  * gst_debug_construct_term_color:
  * @colorinfo: the color info
@@ -357,7 +448,7 @@ gst_debug_construct_term_color (guint colorinfo)
 void
 gst_debug_log_default (GstDebugCategory *category, GstDebugLevel level,
 		       const gchar *file, const gchar *function, gint line,
-		       GObject *object, gchar *message, gpointer unused)
+		       GObject *object, GstDebugMessage *message, gpointer unused)
 {
   gchar *color;
   gchar *clear;
@@ -380,23 +471,15 @@ gst_debug_log_default (GstDebugCategory *category, GstDebugLevel level,
     clear = "";
     pidcolor = g_strdup ("");
   }
-  /* nicely printed object */
-  if (object == NULL) {
-    obj = g_strdup ("");
-  } else if (GST_IS_PAD (object) && GST_OBJECT_NAME (object)) {
-    obj = g_strdup_printf (" [%s:%s]", GST_DEBUG_PAD_NAME (object));
-  } else if (GST_IS_OBJECT (object) && GST_OBJECT_NAME (object)) {
-    obj = g_strdup_printf (" [%s]", GST_OBJECT_NAME (object));
-  } else {
-    obj = g_strdup_printf (" [%s@%p]", G_OBJECT_TYPE_NAME(object), object);  
-  }
+
+  obj = object ? gst_debug_print_object (object) : g_strdup ("");
 
   g_printerr ("%s %s%15s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n", 
   	      gst_debug_level_get_name (level),
               color, gst_debug_category_get_name (category), clear,
               pidcolor, pid, clear,
   	      color, file, line, function, obj, clear,
-  	      message);
+	      gst_debug_message_get (message));
 
   g_free (color);
   g_free (pidcolor);
@@ -739,9 +822,9 @@ _gst_debug_category_new	(gchar *name, guint color, gchar *description)
 }
 /**
  * gst_debug_category_free:
- * @category: category to remove
- * 
- * Removes and frees the category and all associated ressources.
+ * @category: #GstDebugCategory to free.
+ *
+ * Removes and frees the category and all associated resources.
  */
 void
 gst_debug_category_free (GstDebugCategory *category)
@@ -753,23 +836,24 @@ gst_debug_category_free (GstDebugCategory *category)
   __categories = g_slist_remove (__categories, category);
   g_static_mutex_unlock (&__cat_mutex);
 
-  g_free ((gpointer) category->name); 
-  g_free ((gpointer) category->description); 
+  g_free ((gpointer) category->name);
+  g_free ((gpointer) category->description);
   gst_atomic_int_destroy (category->threshold);
   g_free (category->threshold);
   g_free (category);
 }
+
 /**
  * gst_debug_category_set_threshold:
- * @category: category to set threshold for
- * @level: the threshold to set
- * 
+ * @category: a #GstDebugCategory to set threshold of.
+ * @level: the #GstDebugLevel threshold to set.
+ *
  * Sets the threshold of the category to the given level. Debug information will
- * only be output if the threshold is lower or equal to the level of the 
+ * only be output if the threshold is lower or equal to the level of the
  * debugging message.
  * <note><para>
- * Do not use this function in production code, because other functions may 
- * change the threshold of categories as side effect. It is however a nice 
+ * Do not use this function in production code, because other functions may
+ * change the threshold of categories as side effect. It is however a nice
  * function to use when debugging (even from gdb).
  * </para></note>
  */
@@ -780,14 +864,15 @@ gst_debug_category_set_threshold (GstDebugCategory *category, GstDebugLevel leve
 
   gst_atomic_int_set (category->threshold, level);
 }
+
 /**
  * gst_debug_category_reset_threshold:
- * @category: category to set threshold for
- * 
- * Resets the threshold of the category to the default level. Debug information 
- * will only be output if the threshold is lower or equal to the level of the 
+ * @category: a #GstDebugCategory to reset threshold of.
+ *
+ * Resets the threshold of the category to the default level. Debug information
+ * will only be output if the threshold is lower or equal to the level of the
  * debugging message.
- * Use this function to set the threshold back to where it was after using 
+ * Use this function to set the threshold back to where it was after using
  * gst_debug_category_set_threshold().
  */
 void
@@ -795,52 +880,57 @@ gst_debug_category_reset_threshold (GstDebugCategory *category)
 {
   gst_debug_reset_threshold (category, NULL);
 }
+
 /**
  * gst_debug_category_get_threshold:
- * @category: category to get threshold for
- * 
+ * @category: a #GstDebugCategory to get threshold of.
+ *
  * Returns the threshold of a #GstCategory.
- * 
- * Returns: the level that is used as threshold
+ *
+ * Returns: the #GstDebugLevel that is used as threshold.
  */
 GstDebugLevel
 gst_debug_category_get_threshold (GstDebugCategory *category)
 {
   return gst_atomic_int_read (category->threshold);
 }
+
 /**
  * gst_debug_category_get_name:
- * @category: category to get name for
- * 
- * Returns the name of a #GstCategory.
- * 
- * Returns: the name of the category
+ * @category: a #GstDebugCategory to get name of.
+ *
+ * Returns the name of a debug category.
+ *
+ * Returns: the name of the category.
  */
 const gchar *
 gst_debug_category_get_name (GstDebugCategory *category)
 {
   return category->name;
 }
+
 /**
  * gst_debug_category_get_color:
- * @category: category to get color for
- * 
- * Returns the color of a #GstCategory to use when outputting this.
- * 
- * Returns: the color of the category
+ * @category: a #GstDebugCategory to get the color of.
+ *
+ * Returns the color of a debug category used when printing output in this
+ * category.
+ *
+ * Returns: the color of the category.
  */
 guint
 gst_debug_category_get_color (GstDebugCategory *category)
 {
   return category->color;
 }
+
 /**
  * gst_debug_category_get_description:
- * @category: category to get description for
- * 
- * Returns the description of a #GstCategory
- * 
- * Returns: the description of the category
+ * @category: a #GstDebugCategory to get the description of.
+ *
+ * Returns the description of a debug category.
+ *
+ * Returns: the description of the category.
  */
 const gchar *
 gst_debug_category_get_description (GstDebugCategory *category)
@@ -853,9 +943,9 @@ gst_debug_category_get_description (GstDebugCategory *category)
  * Returns a snapshot of a all categories that are currently in use . This list
  * may change anytime.
  * The caller has to free the list after use.
- * <emphasis>This function is not threadsafe, so only use it while only the 
+ * <emphasis>This function is not threadsafe, so only use it while only the
  * main thread is running.</emphasis>
- * 
+ *
  * Returns: the list of categories
  */
 GSList *
@@ -914,6 +1004,36 @@ _gst_debug_register_funcptr (void *ptr, gchar *ptrname)
   return ptr;
 }
 
+#ifdef HAVE_PRINTF_EXTENSION
+static int
+_gst_info_printf_extension (FILE *stream, const struct printf_info *info,
+    const void *const *args)
+{
+  char *buffer;
+  int len;
+  void *ptr;
+
+  buffer = NULL;
+  ptr = *(void **)args[0];
+
+  buffer = gst_debug_print_object (ptr);
+  len = fprintf (stream, "%*s", (info->left ? -info->width : info->width),
+      buffer);
+
+  free (buffer);
+  return len;
+}
+
+static int
+_gst_info_printf_extension_arginfo (const struct printf_info *info, size_t n,
+    int *argtypes)
+{
+  if (n > 0)
+    argtypes[0] = PA_POINTER;
+  return 1;
+}
+#endif /* HAVE_PRINTF_EXTENSION */
+
 #endif /* GST_DISABLE_GST_DEBUG */
 
 #ifdef GST_ENABLE_FUNC_INSTRUMENTATION
