@@ -23,7 +23,12 @@
 #include "gst_private.h"
 
 #include "gstobject.h"
-#include "gstlog.h"
+#include "gstmarshal.h"
+#include "gstinfo.h"
+
+#ifndef GST_DISABLE_TRACE
+#include "gsttrace.h"
+#endif
 
 /* Object signals and args */
 enum {
@@ -38,7 +43,7 @@ enum {
 
 enum {
   ARG_0,
-  ARG_NAME,
+  ARG_NAME
   /* FILL ME */
 };
 
@@ -58,10 +63,16 @@ static GType		gst_signal_object_get_type	(void);
 static void		gst_signal_object_class_init	(GstSignalObjectClass *klass);
 static void		gst_signal_object_init		(GstSignalObject *object);
 
+#ifndef GST_DISABLE_LOADSAVE_REGISTRY
 static guint gst_signal_object_signals[SO_LAST_SIGNAL] = { 0 };
+#endif
 
 static void		gst_object_class_init		(GstObjectClass *klass);
 static void		gst_object_init			(GstObject *object);
+#ifndef GST_DISABLE_TRACE
+static GObject *	gst_object_constructor 		(GType type, guint n_construct_properties, 
+							 GObjectConstructParam *construct_params);
+#endif
 
 static void 		gst_object_set_property 	(GObject * object, guint prop_id, const GValue * value,
 		                                    	 GParamSpec * pspec);
@@ -129,6 +140,7 @@ gst_object_class_init (GstObjectClass *klass)
                   g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1,
                   G_TYPE_OBJECT);
 #ifndef GST_DISABLE_LOADSAVE_REGISTRY
+  /* FIXME This should be the GType of xmlNodePtr instead of G_TYPE_POINTER */
   gst_object_signals[OBJECT_SAVED] =
     g_signal_new ("object_saved", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GstObjectClass, object_saved), NULL, NULL,
@@ -148,12 +160,15 @@ gst_object_class_init (GstObjectClass *klass)
 
   klass->signal_object = g_object_new (gst_signal_object_get_type (), NULL);
 
-  /* see the comments at gst_element_dispatch_properties_changed */
+  /* see the comments at gst_object_dispatch_properties_changed */
   gobject_class->dispatch_properties_changed
 	        = GST_DEBUG_FUNCPTR (gst_object_dispatch_properties_changed);
 
   gobject_class->dispose = gst_object_dispose;
   gobject_class->finalize = gst_object_finalize;
+#ifndef GST_DISABLE_TRACE
+  gobject_class->constructor = gst_object_constructor;
+#endif
 }
 
 static void
@@ -167,6 +182,25 @@ gst_object_init (GstObject *object)
   GST_FLAG_SET (object, GST_FLOATING);
 }
 
+#ifndef GST_DISABLE_TRACE
+static GObject *
+gst_object_constructor (GType type, guint n_construct_properties, GObjectConstructParam *construct_params)
+{
+  const gchar *name;
+  GstAllocTrace *trace;
+  GObject *obj = G_OBJECT_CLASS (parent_class)->constructor (type, n_construct_properties, construct_params);  
+
+  name = g_type_name (type);
+
+  trace = gst_alloc_trace_get (name);
+  if (!trace) {
+    trace = gst_alloc_trace_register (name);
+  }
+  gst_alloc_trace_new (trace, obj);
+  
+  return obj;
+}
+#endif
 /**
  * gst_object_ref:
  * @object: GstObject to reference
@@ -180,8 +214,7 @@ gst_object_ref (GstObject *object)
 {
   g_return_val_if_fail (GST_IS_OBJECT (object), NULL);
 
-  GST_DEBUG (GST_CAT_REFCOUNTING, "ref %p '%s' %d->%d", object,
-	     GST_OBJECT_NAME (object),
+  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "ref %d->%d",
              G_OBJECT (object)->ref_count,
 	     G_OBJECT (object)->ref_count + 1);
 
@@ -202,8 +235,7 @@ gst_object_unref (GstObject *object)
   g_return_if_fail (GST_IS_OBJECT (object));
   g_return_if_fail (G_OBJECT (object)->ref_count > 0);
 
-  GST_DEBUG (GST_CAT_REFCOUNTING, "unref %p '%s' %d->%d", object,
-	     GST_OBJECT_NAME (object),
+  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "unref %d->%d",
              G_OBJECT (object)->ref_count,
 	     G_OBJECT (object)->ref_count - 1);
 
@@ -216,7 +248,7 @@ gst_object_unref (GstObject *object)
  *
  * Removes floating reference on an object.  Any newly created object has
  * a refcount of 1 and is FLOATING.  This function should be used when
- * creating a new object to symbolically 'take ownership of' the object.
+ * creating a new object to symbolically 'take ownership' of the object.
  * Use #gst_object_set_parent to have this done for you.
  */
 void
@@ -225,7 +257,7 @@ gst_object_sink (GstObject *object)
   g_return_if_fail (object != NULL);
   g_return_if_fail (GST_IS_OBJECT (object));
 
-  GST_DEBUG (GST_CAT_REFCOUNTING, "sink %p '%s'", object, GST_OBJECT_NAME (object));
+  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "sink");
 
   if (GST_OBJECT_FLOATING (object)) {
     GST_FLAG_UNSET (object, GST_FLOATING);
@@ -234,7 +266,7 @@ gst_object_sink (GstObject *object)
 }
 
 /**
- * gst_object_swap:
+ * gst_object_replace:
  * @oldobj: pointer to place of old GstObject
  * @newobj: new GstObject
  *
@@ -242,46 +274,28 @@ gst_object_sink (GstObject *object)
  * puts the newobj in *oldobj.
  */
 void
-gst_object_swap (GstObject **oldobj, GstObject *newobj)
+gst_object_replace (GstObject **oldobj, GstObject *newobj)
 {
+  g_return_if_fail (oldobj != NULL);
+  g_return_if_fail (*oldobj == NULL || GST_IS_OBJECT (*oldobj));
+  g_return_if_fail (newobj == NULL || GST_IS_OBJECT (newobj));
+	
+  GST_CAT_LOG (GST_CAT_REFCOUNTING, "replace %s %s", 
+	       *oldobj ? GST_STR_NULL (GST_OBJECT_NAME (*oldobj)) : "(NONE)", 
+	       newobj ? GST_STR_NULL (GST_OBJECT_NAME (newobj)) : "(NONE)");
+
   if (*oldobj != newobj) {
-    if (newobj)
-      gst_object_ref (newobj);
-    if (*oldobj)
-      gst_object_unref (*oldobj);
+    if (newobj)  gst_object_ref (newobj);
+    if (*oldobj) gst_object_unref (*oldobj);
 
     *oldobj = newobj;
   }
 }
 
-/**
- * gst_object_destroy:
- * @object: GstObject to destroy
- *
- * Destroy the object.
- * 
- */
-void
-gst_object_destroy (GstObject *object)
-{
-  g_return_if_fail (object != NULL);
-  g_return_if_fail (GST_IS_OBJECT (object));
-
-  GST_DEBUG (GST_CAT_REFCOUNTING, "destroy %p '%s'", object, GST_OBJECT_NAME (object));
-
-  if (!GST_OBJECT_DESTROYED (object))
-  {
-    /* need to hold a reference count around all class method
-     * invocations.
-     */
-    g_object_run_dispose (G_OBJECT (object));
-  }
-}
-
 static void
 gst_object_dispose (GObject *object)
 {
-  GST_DEBUG (GST_CAT_REFCOUNTING, "dispose %p '%s'", object, GST_OBJECT_NAME (object));
+  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "dispose");
   
   GST_FLAG_SET (GST_OBJECT (object), GST_DESTROYED);
   GST_OBJECT_PARENT (object) = NULL;
@@ -289,27 +303,38 @@ gst_object_dispose (GObject *object)
   parent_class->dispose (object);
 }
 
-/* finilize is called when the object has to free its resources */
+/* finalize is called when the object has to free its resources */
 static void
 gst_object_finalize (GObject *object)
 {
   GstObject *gstobject = GST_OBJECT (object);
 
-  GST_DEBUG (GST_CAT_REFCOUNTING, "finalize %p '%s'", object, GST_OBJECT_NAME (object));
+  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "finalize");
 
   g_signal_handlers_destroy (object);
 
-  if (gstobject->name != NULL)
-    g_free (gstobject->name);
+  g_free (gstobject->name);
 
   g_mutex_free (gstobject->lock);
 
+#ifndef GST_DISABLE_TRACE
+  {
+    const gchar *name;
+    GstAllocTrace *trace;
+  
+    name = g_type_name (G_OBJECT_TYPE (object));
+    trace = gst_alloc_trace_get (name);
+    g_assert (trace);
+    gst_alloc_trace_free (trace, object);
+  }
+#endif
+
   parent_class->finalize (object);
 }
 
-/* Changing a GObject property of an element will result in "deep_notify"
- * signals being emitted by the element itself, as well as in each parent
- * element. This is so that an application can connect a listener to the
+/* Changing a GObject property of a GstObject will result in "deep_notify"
+ * signals being emitted by the object itself, as well as in each parent
+ * object. This is so that an application can connect a listener to the
  * top-level bin to catch property-change notifications for all contained
  * elements. */
 static void
@@ -328,7 +353,7 @@ gst_object_dispatch_properties_changed (GObject     *object,
   while (gst_object) {
     /* need own category? */
     for (i = 0; i < n_pspecs; i++) {
-      GST_DEBUG (GST_CAT_EVENT, "deep notification from %s to %s (%s)", GST_OBJECT_NAME (object),
+      GST_CAT_LOG (GST_CAT_EVENT, "deep notification from %s to %s (%s)", GST_OBJECT_NAME (object),
                  GST_OBJECT_NAME (gst_object), pspecs[i]->name);
       g_signal_emit (gst_object, gst_object_signals[DEEP_NOTIFY], g_quark_from_string (pspecs[i]->name),
                      (GstObject *) object, pspecs[i]);
@@ -407,8 +432,10 @@ gst_object_set_name_default (GstObject *object)
    * may ever assign a name */
   G_LOCK (object_name_mutex);
 
-  if (!object_name_counts)
-    object_name_counts = g_hash_table_new (g_str_hash, g_str_equal);
+  if (!object_name_counts) {
+    object_name_counts = g_hash_table_new_full (g_str_hash, g_str_equal,
+        g_free, NULL);
+  }
 
   count = GPOINTER_TO_INT (g_hash_table_lookup (object_name_counts, type_name));
   g_hash_table_insert (object_name_counts, g_strdup (type_name), 
@@ -471,9 +498,10 @@ gst_object_get_name (GstObject *object)
  * @object: GstObject to set parent of
  * @parent: new parent of object
  *
- * Set the parent of the object.  The object's reference count is
- * incremented.
- * signals the parent-set signal
+ * Sets the parent of @object. The object's reference count will be incremented,
+ * and any floating reference will be removed (see gst_object_sink()).
+ *
+ * Causes the parent-set signal to be emitted.
  */
 void
 gst_object_set_parent (GstObject *object, GstObject *parent)
@@ -483,11 +511,7 @@ gst_object_set_parent (GstObject *object, GstObject *parent)
   g_return_if_fail (parent != NULL);
   g_return_if_fail (GST_IS_OBJECT (parent));
   g_return_if_fail (object != parent);
-
-  if (object->parent != NULL) {
-    GST_ERROR_OBJECT (object,object->parent, "object's parent is already set, must unparent first");
-    return;
-  }
+  g_return_if_fail (object->parent == NULL);
 
   gst_object_ref (object);
   gst_object_sink (object);
@@ -500,7 +524,7 @@ gst_object_set_parent (GstObject *object, GstObject *parent)
  * gst_object_get_parent:
  * @object: GstObject to get parent of
  *
- * Return the parent of the object.
+ * Returns the parent of @object.
  *
  * Returns: parent of the object
  */
@@ -517,7 +541,7 @@ gst_object_get_parent (GstObject *object)
  * gst_object_unparent:
  * @object: GstObject to unparent
  *
- * Clear the parent of the object, removing the associated reference.
+ * Clear the parent of @object, removing the associated reference.
  */
 void
 gst_object_unparent (GstObject *object)
@@ -527,7 +551,7 @@ gst_object_unparent (GstObject *object)
   if (object->parent == NULL)
     return;
 
-  GST_DEBUG (GST_CAT_REFCOUNTING, "unparent '%s'",GST_OBJECT_NAME(object));
+  GST_CAT_LOG_OBJECT (GST_CAT_REFCOUNTING, object, "unparent");
   
   g_signal_emit (G_OBJECT (object), gst_object_signals[PARENT_UNSET], 0, object->parent);
 
@@ -540,11 +564,9 @@ gst_object_unparent (GstObject *object)
  * @list: a list of #GstObject to check through
  * @name: the name to search for
  *
- * This function checks through the list of objects to see if the name
- * given appears in the list as the name of an object.  It returns TRUE if
- * the name does not exist in the list.
+ * Checks to see if there is any object named @name in @list.
  *
- * Returns: TRUE if the name doesn't appear in the list, FALSE if it does.
+ * Returns: TRUE if the name does not appear in the list, FALSE if it does.
  */
 gboolean
 gst_object_check_uniqueness (GList *list, const gchar *name)
@@ -673,7 +695,7 @@ gst_object_get_property (GObject* object, guint prop_id,
  * @object: GstObject to get the path from
  *
  * Generates a string describing the path of the object in
- * the object hierarchy. Only useful (or used) for debugging
+ * the object hierarchy. Only useful (or used) for debugging.
  *
  * Returns: a string describing the path of the object
  */
