@@ -21,10 +21,13 @@
  */
 
 #include "gst_private.h"
+
 #include "gstpluginfeature.h"
 #include "gstplugin.h"
 #include "gstregistry.h"
-#include "gstlog.h"
+#include "gstinfo.h"
+
+#include <string.h>
 
 static void		gst_plugin_feature_class_init		(GstPluginFeatureClass *klass);
 static void		gst_plugin_feature_init			(GstPluginFeature *feature);
@@ -92,14 +95,16 @@ gst_plugin_feature_ensure_loaded (GstPluginFeature *feature)
   plugin = (GstPlugin *) (feature->manager);
 
   if (plugin && !gst_plugin_is_loaded (plugin)) {
+#ifndef GST_DISABLE_REGISTRY 
     if (GST_IS_REGISTRY (plugin->manager)) {
-      GST_DEBUG (GST_CAT_PLUGIN_LOADING, 
-	         "loading plugin %s for feature", plugin->name);
+      GST_CAT_DEBUG (GST_CAT_PLUGIN_LOADING, 
+	         "loading plugin %s for feature", plugin->desc.name);
 
       if (gst_registry_load_plugin (GST_REGISTRY (plugin->manager), plugin) != GST_REGISTRY_OK)
 	return FALSE;
     }
     else
+#endif /* GST_DISABLE_REGISTRY */
       return FALSE;
   }
   return TRUE;
@@ -126,5 +131,78 @@ gst_plugin_feature_unload_thyself (GstPluginFeature *feature)
     oclass->unload_thyself (feature);
 }
 
+gboolean
+gst_plugin_feature_type_name_filter (GstPluginFeature *feature,
+		                     GstTypeNameData *data)
+{
+  return ((data->type == 0    || data->type == G_OBJECT_TYPE (feature)) &&
+          (data->name == NULL || !strcmp (data->name, GST_PLUGIN_FEATURE_NAME (feature))));
+}
+/**
+ * gst_plugin_feature_set_rank:
+ * @feature: feature to rank
+ * @rank: rank value - higher number means more priority rank
+ *
+ * Specifies a rank for a plugin feature, so that autoplugging uses
+ * the most appropriate feature.
+ */
+void
+gst_plugin_feature_set_rank (GstPluginFeature *feature, guint rank)
+{
+  g_return_if_fail (feature != NULL);
+  g_return_if_fail (GST_IS_PLUGIN_FEATURE (feature));
 
+  feature->rank = rank;
+}
+/**
+ * gst_plugin_feature_set_name:
+ * @feature: a feature
+ * @name: the name to set
+ *
+ * Sets the name of a plugin feature. The name uniquely identifies a feature
+ * within all features of the same type. Renaming a plugin feature is not 
+ * allowed.
+ */
+void
+gst_plugin_feature_set_name (GstPluginFeature *feature, const gchar *name)
+{
+  g_return_if_fail (GST_IS_PLUGIN_FEATURE (feature));
+  g_return_if_fail (name != NULL);
+
+  if (feature->name) {
+    g_return_if_fail (strcmp (feature->name, name) == 0);
+  } else {
+    feature->name = g_strdup (name);
+  }
+}
+/**
+ * gst_plugin_feature_get rank:
+ * @feature: a feature
+ *
+ * Gets the rank of a plugin feature.
+ *
+ * Returns: The rank of the feature
+ */
+guint
+gst_plugin_feature_get_rank (GstPluginFeature *feature)
+{
+  g_return_val_if_fail (GST_IS_PLUGIN_FEATURE (feature), GST_RANK_NONE);
+  
+  return feature->rank;
+}
+/**
+ * gst_plugin_feature_get_name:
+ * @feature: a feature
+ *
+ * Gets the name of a plugin feature.
+ *
+ * Returns: the name
+ */
+G_CONST_RETURN gchar *
+gst_plugin_feature_get_name (GstPluginFeature *feature)
+{
+  g_return_val_if_fail (GST_IS_PLUGIN_FEATURE (feature), NULL);
+  
+  return feature->name;
+}
 
