@@ -51,6 +51,25 @@ static GHashTable *__tags;
 #define TAG_LOCK g_mutex_lock (__tag_mutex)
 #define TAG_UNLOCK g_mutex_unlock (__tag_mutex)
 
+GType
+gst_tag_list_get_type (void)
+{
+  static GType _gst_tag_list_type;
+
+  if (_gst_tag_list_type == 0) {
+    _gst_tag_list_type = g_boxed_type_register_static ("GstTagList",
+        (GBoxedCopyFunc) gst_tag_list_copy,
+        (GBoxedFreeFunc) gst_tag_list_free);
+
+#if 0
+    g_value_register_transform_func(_gst_tag_list_type, G_TYPE_STRING,
+        _gst_structure_transform_to_string);
+#endif
+  }
+
+  return _gst_tag_list_type;
+}
+
 void
 _gst_tag_initialize (void)
 {
@@ -97,6 +116,16 @@ _gst_tag_initialize (void)
 		    _("track count"),
 		    _("count of tracks inside collection this track belongs to"), 
 		    gst_tag_merge_use_first);
+  gst_tag_register (GST_TAG_ALBUM_VOLUME_NUMBER, GST_TAG_FLAG_META,
+		    G_TYPE_UINT,
+		    _("disc number"),
+		    _("disc number inside a collection"),
+		    gst_tag_merge_use_first);
+  gst_tag_register (GST_TAG_ALBUM_VOLUME_COUNT, GST_TAG_FLAG_META,
+		    G_TYPE_UINT,
+		    _("disc count"),
+		    _("count of discs inside collection this disc belongs to"), 
+		    gst_tag_merge_use_first);
   gst_tag_register (GST_TAG_LOCATION, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("location"),
@@ -132,7 +161,7 @@ _gst_tag_initialize (void)
 		    _("contact"),
 		    _("contact information"),
 		    gst_tag_merge_strings_with_comma);
-  gst_tag_register (GST_TAG_LICENSE,	 GST_TAG_FLAG_META,
+  gst_tag_register (GST_TAG_LICENSE, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("license"),
 		    _("license of data"),
@@ -152,6 +181,16 @@ _gst_tag_initialize (void)
 		    _("codec"),
 		    _("codec the data is stored in"),
 		    gst_tag_merge_strings_with_comma);
+  gst_tag_register (GST_TAG_VIDEO_CODEC, GST_TAG_FLAG_ENCODED,
+		    G_TYPE_STRING,
+		    _("video codec"),
+		    _("codec the video data is stored in"),
+		    NULL);
+  gst_tag_register (GST_TAG_AUDIO_CODEC, GST_TAG_FLAG_ENCODED,
+		    G_TYPE_STRING,
+		    _("audio codec"),
+		    _("codec the audio data is stored in"),
+		    NULL);
   gst_tag_register (GST_TAG_BITRATE, GST_TAG_FLAG_ENCODED,
 		    G_TYPE_UINT,
 		    _("bitrate"),
@@ -924,7 +963,3 @@ TAG_MERGE_FUNCS (double, gdouble)
   
 #define COPY_FUNC g_strdup
 TAG_MERGE_FUNCS (string, gchar *)
-
-
-
-
