@@ -46,140 +46,16 @@ G_STMT_START{                                                   \
 #endif
 
 static void
-print_prop (GstPropsEntry *prop, gint pfx)
+print_caps (const GstCaps *caps, gint pfx)
 {
-  GstPropsType type;
+  char *s;
 
-  type = gst_props_entry_get_props_type (prop);
-
-  switch (type) {
-    case GST_PROPS_INT_TYPE:
-    {
-      gint val;
-      gst_props_entry_get_int (prop, &val);
-      PUT_STRING (pfx, "<int name=\"%s\" value=\"%d\"/>",
-                      gst_props_entry_get_name (prop), val);
-      break;
-    }
-    case GST_PROPS_INT_RANGE_TYPE:
-    {
-      gint min, max;
-      gst_props_entry_get_int_range (prop, &min, &max);
-      PUT_STRING (pfx, "<range name=\"%s\" min=\"%d\" max=\"%d\"/>",
-                      gst_props_entry_get_name (prop), min, max);
-      break;
-    }
-    case GST_PROPS_FLOAT_TYPE:
-    {
-      gfloat val;
-      gst_props_entry_get_float (prop, &val);
-      PUT_STRING (pfx, "<float name=\"%s\" value=\"%f\"/>",
-                      gst_props_entry_get_name (prop), val);
-      break;
-    }
-    case GST_PROPS_FLOAT_RANGE_TYPE:
-    {
-      gfloat min, max;
-      gst_props_entry_get_float_range (prop, &min, &max);
-      PUT_STRING (pfx, "<floatrange name=\"%s\" min=\"%f\" max=\"%f\"/>",
-                      gst_props_entry_get_name (prop), min, max);
-      break;
-    }
-    case GST_PROPS_BOOLEAN_TYPE:
-    {
-      gboolean val;
-      gst_props_entry_get_boolean (prop, &val);
-      PUT_STRING (pfx, "<boolean name=\"%s\" value=\"%s\"/>",
-                      gst_props_entry_get_name (prop), val ? "true" : "false");
-      break;
-    }
-    case GST_PROPS_STRING_TYPE:
-    {
-      const gchar *val;
-      gst_props_entry_get_string (prop, &val);
-      PUT_STRING (pfx, "<string name=\"%s\" value=\"%s\"/>",
-                      gst_props_entry_get_name (prop), val);
-      break;
-    }
-    case GST_PROPS_FOURCC_TYPE:
-    {
-      guint32 val;
-      gst_props_entry_get_fourcc_int (prop, &val);
-      PUT_STRING (pfx, "<!--%c%c%c%c-->",
-             (gchar)( val        & 0xff),
-             (gchar)((val >> 8)  & 0xff),
-             (gchar)((val >> 16) & 0xff),
-             (gchar)((val >> 24) & 0xff));
-      PUT_STRING (pfx, "<fourcc name=\"%s\" hexvalue=\"%08x\"/>",
-                      gst_props_entry_get_name (prop), val);
-      break;
-    }
-    case GST_PROPS_LIST_TYPE:
-    {
-      const GList *list;
-
-      gst_props_entry_get_list (prop, &list);
-      PUT_STRING (pfx, "<list name=\"%s\">", gst_props_entry_get_name (prop));
-      while (list) {
-        GstPropsEntry *listentry;
-
-        listentry = (GstPropsEntry*) (list->data);
-        print_prop (listentry, pfx + 1);
-
-        list = g_list_next (list);
-      }
-      PUT_END_TAG (pfx, "list");
-      break;
-    }
-    default:
-      break;
-  }
-}
-
-static void
-print_props (GstProps *properties, gint pfx)
-{
-  GList *props;
-  GstPropsEntry *prop;
-
-  props = properties->properties;
-
-  if (!props)
-    return;
-
-  PUT_START_TAG (pfx, "properties");
-
-  while (props) {
-    prop = (GstPropsEntry*) (props->data);
-    props = g_list_next (props);
-
-    print_prop (prop, pfx + 1);
-  }
-  PUT_END_TAG (pfx, "properties");
-}
-
-static void
-print_caps (GstCaps *caps, gint pfx)
-{
   if (!caps)
     return;
 
-  PUT_START_TAG (pfx, "capscomp");
-
-  while (caps) {
-    PUT_START_TAG (pfx + 1, "caps");
-    PUT_ESCAPED (pfx + 2, "name", caps->name);
-
-    PUT_ESCAPED (pfx + 2, "type", gst_caps_get_mime (caps));
-
-    if (caps->properties) {
-      print_props(caps->properties, pfx + 2);
-    }
-    PUT_END_TAG (pfx + 1, "caps");
-
-    caps = caps->next;
-  }
-  PUT_END_TAG (pfx, "capscomp");
+  s = gst_caps_to_string (caps);
+  PUT_ESCAPED (pfx, "caps", s);
+  g_free(s);
 }
 
 static void
@@ -193,7 +69,7 @@ print_formats (const GstFormat *formats, gint pfx)
       PUT_STRING (pfx, "<format id=\"%d\" nick=\"%s\">%s</format>",
                   *formats, definition->nick, definition->description);
     else
-      PUT_STRING (pfx, "<format id=\"%d\">unkown</format>",
+      PUT_STRING (pfx, "<format id=\"%d\">unknown</format>",
                   *formats);
 
     formats++;
@@ -211,7 +87,7 @@ print_query_types (const GstQueryType *types, gint pfx)
       PUT_STRING (pfx, "<query-type id=\"%d\" nick=\"%s\">%s</query-type>",
                   *types, definition->nick, definition->description);
     else
-      PUT_STRING (pfx, "<query-type id=\"%d\">unkown</query-type>",
+      PUT_STRING (pfx, "<query-type id=\"%d\">unknown</query-type>",
                   *types);
 
     types++;
@@ -768,9 +644,9 @@ print_element_info (GstElementFactory *factory)
         PUT_STRING (4, "<intlink-function function=\"%s\"/>",
                 GST_DEBUG_FUNCPTR_NAME(realpad->intlinkfunc));
 
-      if (realpad->bufferpoolfunc)
-        PUT_STRING (4, "<bufferpool-function function=\"%s\"/>",
-                GST_DEBUG_FUNCPTR_NAME(realpad->bufferpoolfunc));
+      if (realpad->bufferallocfunc)
+        PUT_STRING (4, "<bufferalloc-function function=\"%s\"/>",
+                GST_DEBUG_FUNCPTR_NAME(realpad->bufferallocfunc));
       PUT_END_TAG (3, "implementation");
 
       if (realpad->caps) {
@@ -828,15 +704,6 @@ print_element_list (void)
         g_print ("%s:  %s: %s\n", plugin->desc.name,
                 GST_PLUGIN_FEATURE_NAME (factory) ,factory->details.longname);
       }
-#ifndef GST_DISABLE_AUTOPLUG
-      else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
-        GstAutoplugFactory *factory;
-
-        factory = GST_AUTOPLUG_FACTORY (feature);
-        g_print ("%s:  %s: %s\n", plugin->desc.name,
-                GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
-      }
-#endif
 #ifndef GST_DISABLE_INDEX
       else if (GST_IS_INDEX_FACTORY (feature)) {
         GstIndexFactory *factory;
@@ -867,16 +734,6 @@ print_element_list (void)
         g_print ("%s:  %s: %s\n", plugin->desc.name,
                 GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       }
-#ifndef GST_DISABLE_URI
-      else if (GST_IS_URI_HANDLER (feature)) {
-        GstURIHandler *handler;
-
-        handler = GST_URI_HANDLER (feature);
-        g_print ("%s:  %s: \"%s\" (%s) element \"%s\" property \"%s\"\n",
-                 plugin->desc.name, GST_PLUGIN_FEATURE_NAME (handler), handler->uri,
-                 handler->longdesc, handler->element, handler->property);
-      }
-#endif
       else {
         g_print ("%s:  %s (%s)\n", plugin->desc.name,
                 GST_PLUGIN_FEATURE_NAME (feature),
@@ -906,7 +763,6 @@ print_plugin_info (GstPlugin *plugin)
   g_print ("  Filename:\t%s\n",	    plugin->filename);
   g_print ("  Version:\t%s\n",	    plugin->desc.version);
   g_print ("  License:\t%s\n",	    plugin->desc.license);
-  g_print ("  Copyright:\t%s\n",    plugin->desc.copyright);
   g_print ("  Package:\t%s\n",	    plugin->desc.package);
   g_print ("  Origin URL:\t%s\n",   plugin->desc.origin);
   g_print ("\n");
@@ -926,15 +782,6 @@ print_plugin_info (GstPlugin *plugin)
               factory->details.longname);
       num_elements++;
     }
-#ifndef GST_DISABLE_AUTOPLUG
-    else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
-      GstAutoplugFactory *factory;
-
-      factory = GST_AUTOPLUG_FACTORY (feature);
-      g_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
-      num_autoplug++;
-    }
-#endif
 #ifndef GST_DISABLE_INDEX
     else if (GST_IS_INDEX_FACTORY (feature)) {
       GstIndexFactory *factory;
@@ -1052,14 +899,6 @@ main (int argc, char *argv[])
            g_print ("%s: an index\n", argv[1]);
            return 0;
          }
-#endif
-#ifndef GST_DISABLE_AUTOPLUG
-         feature = gst_registry_pool_find_feature (argv[1],
-                                                   GST_TYPE_AUTOPLUG_FACTORY);
-         if (feature) {
-           g_print ("%s: an autoplugger\n", argv[1]);
-           return 0;
-         }
 #endif
          feature = gst_registry_pool_find_feature (argv[1],
                                                    GST_TYPE_TYPE_FIND_FACTORY);
